﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using nicoAE.Plugin.Interface;
using nicoAE.Plugin.Controls;
using nicoAE.Plugin.Property;
using nicoAE.Plugin.Utils;
using nicoAE._3DUtils;
using System.Windows.Forms;

namespace ExtendableEditControlTest
{
    /// <summary>
    /// 入れ子コントロールサンプルエフェクト。
    /// </summary>
    public class SampleEffect : IEffect, IViolateImageEffect
    {
        IHost ho = null;
        RenderingContext rc = null;
        Size rcSize = new Size();

        #region IEffect メンバ

        public string PluginName
        {
            get
            {
                return "入れ子プロパティテスト";
            }
        }

        public string Category
        {
            get
            {
                return "サンプル";
            }
        }

        public MediaType CorrespondingType
        {
            get
            {
                return MediaType.Image | MediaType.Video;
            }
        }

        public bool PresetProperty
        {
            get
            {
                return false;
            }
        }

        public void Initialize(IHost host, MediaInfo info)
        {
            ho = host;
            //レンダリングコンテキストを作成。
            rc = new RenderingContext(host.ProjectSize);
            rcSize = host.ProjectSize;
            if (rc.Success)
            {
                rc.SetCurrent();
                gl.Enable(GLFlags.GL_DEPTH_TEST);
                gl.DepthFunc(GLFlags.GL_LEQUAL);
                gl.AlphaFunc(GLFlags.GL_GREATER, 0);
                gl.Enable(GLFlags.GL_ALPHA_TEST);
                gl.Enable(GLFlags.GL_BLEND);
                gl.MatrixMode(GLFlags.GL_PROJECTION);
                gl.LoadIdentity();
                glu.Perspective(61.5, 1, 1, int.MaxValue);
                rc.SetNullCurrent();
            }
            else
            {
                MessageBox.Show("レンダリングコンテキストの作成に失敗しました。OpenGLが使用できない可能性があります。", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                rc.Dispose();
                rc = null;
            }
        }

        public IPropertyEditControl[] GetControls()
        {
            return new IPropertyEditControl[] { new VertexEditControl("アンカーポイント", "X座標", "Y座標", "Z座標"), new VertexEditControl("位置", "X座標", "Y座標", "Z座標"), new VertexRotationEditControl("回転", "X回転", "Y回転", "Z回転"), new VertexEditControl("スケール", "X座標", "Y座標", "Z座標"), new VertexColorArrayEditControl("頂点", 1) };
        }

        public IEffectProperty[] GetDefaultProperty()
        {
            VertexColorProperty[] vertexs = new VertexColorProperty[4];
            vertexs[0] = new VertexColorProperty("VertexColorPropertyの", new Vertex(-1, -1, 0), Color.Blue);
            vertexs[1] = new VertexColorProperty("名前は", new Vertex(-1, 1, 0), Color.Red);
            vertexs[2] = new VertexColorProperty("セットするときに無視されるので", new Vertex(1, 1, 0), Color.Green);
            vertexs[3] = new VertexColorProperty("何でもOK", new Vertex(1, -1, 0), Color.White);
            return new IEffectProperty[] { new VertexProperty("アンカーポイント", new Vertex(), new Vertex(int.MaxValue, int.MaxValue, int.MaxValue), new Vertex(int.MinValue, int.MinValue, int.MinValue)), new VertexProperty("位置", new Vertex(), new Vertex(int.MaxValue, int.MaxValue, int.MaxValue), new Vertex(int.MinValue, int.MinValue, int.MinValue)), new VertexRotationProperty("回転", 0, 0, 0, 0, 0, 0), new VertexProperty("スケール", new Vertex(100, 100, 100), new Vertex(int.MaxValue, int.MaxValue, int.MaxValue), new Vertex(int.MinValue, int.MinValue, int.MinValue)), new VertexColorArrayProperty("頂点", vertexs) };
        }

        public KeyFrame[] GetPresetProperty(int index, string propertyName)
        {
            throw new NotImplementedException();
        }

        public byte[] ProcessingAudio(byte[] audio, IEffectProperty[] property)
        {
            throw new NotImplementedException();
        }

        public Bitmap ProcessingImage(Bitmap image, IEffectProperty[] property)
        {
            //IViolateImageEffectを実装しているので無視。
            throw new NotImplementedException();
        }

        #endregion

        #region IDisposable メンバ

        public void Dispose()
        {
            if (rc != null)
            {
                rc.Dispose();
            }
        }

        #endregion

        #region IViolateImageEffect メンバ

        public AnchorImage ProcessingImage(AnchorImage image, IEffectProperty[] property)
        {
            //OpenGLが使えない場合はそのまま入力を返す。
            if (rc != null)
            {
                Vertex anchor = new Vertex();
                Vertex pos = new Vertex();
                Vertex scale = new Vertex();
                VertexRotation rotate = new VertexRotation();
                VertexColorArrayProperty vertexs = null;
                for (int i = 0; i < property.Length; i++)
                {
                    switch (property[i].PropertyName)
                    {
                        case "アンカーポイント":
                            {
                                anchor = ((VertexProperty)property[i]).TypeSafeValue;
                                break;
                            }
                        case "位置":
                            {
                                pos = ((VertexProperty)property[i]).TypeSafeValue;
                                break;
                            }
                        case "回転":
                            {
                                rotate = ((VertexRotationProperty)property[i]).TypeSafeValue;
                                break;
                            }
                        case "スケール":
                            {
                                scale = ((VertexProperty)property[i]).TypeSafeValue;
                                break;
                            }
                        case "頂点":
                            {
                                vertexs = (VertexColorArrayProperty)property[i];
                                break;
                            }
                    }
                }
                //全く使用しないため、リーク防止
                image.Dispose();

                //コンポジションサイズが変わっていた場合はコンテキストを作り直す。
                if (!rcSize.Equals(ho.ProjectSize))
                {
                    rc.Dispose();
                    rcSize = ho.ProjectSize;
                    rc = new RenderingContext(rcSize);
                    if (rc.Success)
                    {
                        rc.SetCurrent();
                        gl.Enable(GLFlags.GL_DEPTH_TEST);
                        gl.DepthFunc(GLFlags.GL_LEQUAL);
                        gl.AlphaFunc(GLFlags.GL_GREATER, 0);
                        gl.Enable(GLFlags.GL_ALPHA_TEST);
                        gl.Enable(GLFlags.GL_BLEND);
                        gl.MatrixMode(GLFlags.GL_PROJECTION);
                        gl.LoadIdentity();
                        glu.Perspective(61.5, 1, 1, int.MaxValue);
                        rc.SetNullCurrent();
                    }
                    else
                    {
                        MessageBox.Show("レンダリングコンテキストの作成に失敗しました。OpenGLが使用できない可能性があります。", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        rc.Dispose();
                        rc = null;
                        return image;
                    }
                }
                rc.SetCurrent();
                //各バッファをクリア。
                gl.Clear(GLFlags.GL_COLOR_BUFFER_BIT | GLFlags.GL_DEPTH_BUFFER_BIT);
                gl.ClearColor(0, 0, 0, 0);

                //変換行列を初期化。
                gl.MatrixMode(GLFlags.GL_MODELVIEW);
                gl.LoadIdentity();
                //面倒なのでカメラは固定。
                glu.LookAt(0.0, 0.0, 2.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

                //比率合わせ
                double hfac = (double)rcSize.Width / (double)rcSize.Height;

                Matrix4D matrix = new Matrix4D();
                matrix.Translate(-anchor.X, -anchor.Y, -anchor.Z);
                matrix.Rotate(rotate.XAngle, rotate.YAngle, rotate.ZAngle);
                matrix.Scale(scale.X / 100.0, scale.Y / 100.0, scale.Z / 100.0);
                matrix.Translate(pos.X, pos.Y, pos.Z);
                matrix.Scale(1.0, hfac, 1.0);
                gl.MultMatrix(matrix.GetMatrix());

                //描画開始。
                gl.Begin(GLFlags.GL_POLYGON);
                for (int i = 0; i < vertexs.Vertexs.Length; i++)
                {
                    Color c = vertexs[i].Color;
                    Vertex v = vertexs[i].Vertex;
                    gl.Color4d(c.R / 255.0, c.G / 255.0, c.B / 255.0, c.A / 255.0);
                    gl.Vertex3d(v.X, v.Y, v.Z);
                }
                //描画終了。
                gl.End();
                gl.Flush();
                rc.SetNullCurrent();

                return new AnchorImage(rc.GetImage(rcSize), new PointF(), rcSize);
            }
            else
            {
                return image;
            }
        }

        #endregion
    }
}
