﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using nicoAE.Plugin.Interface;
using nicoAE.Plugin.Controls;
using nicoAE.Plugin.Property;
using nicoAE.Plugin.Utils;
using nicoAE._3DUtils;

namespace ExtendableEditControlTest
{
    /// <summary>
    /// 複数のVertexColorEditControlを格納するコントロール
    /// </summary>
    public partial class VertexColorArrayEditControl : ExtendableEditControl, IPropertyEditControl, IExtendEvent, IScalingTimeline, ISynchronizeKeyFrame
    {
        //IPropertyEditControlが実装してあれば、入れ子の入れ子も可能。
        //その他詳細はVertexColorEditControl参照。

        /// <summary>
        /// exVertexは拡張する頂点の数。
        /// </summary>
        /// <param name="name"></param>
        /// <param name="exVertex">拡張する頂点の数</param>
        public VertexColorArrayEditControl(string name, int exVertex)
        {
            InitializeComponent();
            PropertyName = name;
            //最低3つ。
            vertexs = new VertexColorEditControl[3 + exVertex];
            for (int i = 0; i < vertexs.Length; i++)
            {
                vertexs[i] = new VertexColorEditControl(name + " " + (i + 1).ToString());
                base.AddControl(vertexs[i]);
            }
        }

        /// <summary>
        /// プロパティの名前。
        /// </summary>
        string n = "";
        /// <summary>
        /// 使用するVertexColorEditControlの配列。
        /// </summary>
        VertexColorEditControl[] vertexs = null;

        #region IPropertyEditControl メンバ

        public override string PropertyName
        {
            get
            {
                return n;
            }
            set
            {
                n = value;
                base.PropertyName = value;
            }
        }

        public KeyFrame[] GetKeyFrames()
        {
            //そのままだと1つ分のプロパティのKeyFrameしか保存できないので、KeyFrameContainerに入れる。
            KeyFrameContainer container = new KeyFrameContainer(n);
            for (int i = 0; i < vertexs.Length; i++)
            {
                container.AddKeyFrames(vertexs[i].PropertyName, vertexs[i].GetKeyFrames());
            }
            KeyFrame k = new KeyFrame();
            k.Property = container;
            return new KeyFrame[] { k };
        }

        public IEffectProperty GetProperty(int index)
        {
            VertexColorProperty[] ps = new VertexColorProperty[vertexs.Length];
            for (int i = 0; i < vertexs.Length; i++)
            {
                ps[i] = (VertexColorProperty)vertexs[i].GetProperty(index);
            }
            return new VertexColorArrayProperty(n, ps);
        }

        public void SetDefaultProperty(IEffectProperty property)
        {
            VertexColorArrayProperty p = (VertexColorArrayProperty)property;
            //最低でも3つ以上あればOKとする。
            if (p.Vertexs.Length > 3)
            {
                int min = Math.Min(p.Vertexs.Length, vertexs.Length);
                for (int i = 0; i < min; i++)
                {
                    vertexs[i].SetDefaultProperty(p.Vertexs[i]);
                }
                //名前は無視。
                VertexColorProperty def = new VertexColorProperty("", new Vertex(), Color.White);
                for (int i = min; i < vertexs.Length; i++)
                {
                    vertexs[i].SetDefaultProperty(def);
                }
            }
            else
            {
                //英語が怪しいのはご愛敬。
                throw new ArgumentException("Property length less than 3", "property");
            }
        }

        public void SetKeyFrames(KeyFrame[] frames)
        {
            if (frames.Length > 0)
            {
                KeyFrameContainer container = (KeyFrameContainer)frames[0].Property;
                for (int i = 0; i < vertexs.Length; i++)
                {
                    if (container.ContainsKey(vertexs[i].PropertyName))
                    {
                        vertexs[i].SetKeyFrames(container.GetKeyFrames(vertexs[i].PropertyName));
                    }
                }
            }
        }

        #endregion
    }
}
