﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using nicoAE.Plugin.Interface;
using nicoAE.Plugin.Controls;
using nicoAE.Plugin.Property;
using nicoAE.Plugin.Utils;
using nicoAE._3DUtils;

namespace ExtendableEditControlTest
{
    /// <summary>
    /// VertexColorPropertyを操作するためのコントロール。
    /// </summary>
    public partial class VertexColorEditControl : ExtendableEditControl, IPropertyEditControl, IExtendEvent, IScalingTimeline, ISynchronizeKeyFrame
    {
        //継承するクラスはExtendableEditControl、実装するインターフェースはIPropertyEditControl, IExtendEvent, IScalingTimeline, ISynchronizeKeyFrame。
        //ただし、ExtendableEditControlにIExtendEvent, IScalingTimeline, ISynchronizeKeyFrameのメソッドは実装済なので、宣言するだけでOK。

        public VertexColorEditControl(string name)
        {
            InitializeComponent();
            PropertyName = name;
            vertex = new VertexEditControl("頂点", "X座標", "Y座標", "Z座標");
            color = new ColorEditControl("頂点カラー");
            //ExtendableEditControlに使用するIPropertyControlを追加する。
            base.AddControl(vertex);
            base.AddControl(color);
        }

        /// <summary>
        /// プロパティの名前。
        /// </summary>
        string n = "";
        /// <summary>
        /// 頂点操作用のコントロール。
        /// </summary>
        VertexEditControl vertex = null;
        /// <summary>
        /// 頂点カラー操作用のコントロール。
        /// </summary>
        ColorEditControl color = null;

        #region IPropertyEditControl メンバ

        //このサンプルではオーバーライドしているが、しなくてもOK。
        public override string PropertyName
        {
            get
            {
                return n;
            }
            set
            {
                n = value;
                //ExtendableEditControlのラベルに名前を表示するために必ず代入する。
                base.PropertyName = value;
            }
        }

        public KeyFrame[] GetKeyFrames()
        {
            //そのままだと1つ分のプロパティのKeyFrameしか保存できないので、KeyFrameContainerに入れる。
            KeyFrameContainer container = new KeyFrameContainer(n);
            container.AddKeyFrames(vertex.PropertyName, vertex.GetKeyFrames());
            container.AddKeyFrames(color.PropertyName, color.GetKeyFrames());
            KeyFrame k = new KeyFrame();
            k.Property = container;
            return new KeyFrame[] { k };
        }

        public IEffectProperty GetProperty(int index)
        {
            //それぞれのプロパティを取得してからVertexColorPropertyに代入する。
            VertexProperty v = (VertexProperty)vertex.GetProperty(index);
            ColorProperty c = (ColorProperty)color.GetProperty(index);
            return new VertexColorProperty(n, v.TypeSafeValue, c.TypeSafeValue);
        }

        public void SetDefaultProperty(IEffectProperty property)
        {
            //そのままだと各IPropertyControlにデフォルトのプロパティをセットできないので、分解する。
            VertexColorProperty def = (VertexColorProperty)property;
            vertex.SetDefaultProperty(new VertexProperty(vertex.PropertyName, def.Vertex, new Vertex(double.MaxValue, double.MaxValue, double.MaxValue), new Vertex(double.MinValue, double.MinValue, double.MinValue)));
            color.SetDefaultProperty(new ColorProperty(color.PropertyName, def.Color));
        }

        public void SetKeyFrames(KeyFrame[] frames)
        {
            if (frames.Length > 0)
            {
                KeyFrameContainer container = (KeyFrameContainer)frames[0].Property;
                //各プロパティのKeyFrameが格納されているかどうかを一応調べる。
                if (container.ContainsKey(vertex.PropertyName))
                {
                    vertex.SetKeyFrames(container.GetKeyFrames(vertex.PropertyName));
                }
                if (container.ContainsKey(color.PropertyName))
                {
                    color.SetKeyFrames(container.GetKeyFrames(color.PropertyName));
                }
            }
        }

        #endregion
    }
}
